<?php
/**
 * Copyright 2018 Kacper Donat
 *
 * @author Kacper "Kadet" Donat <kacper@kadet.net>
 *
 * Full license available in separate LICENSE file
 */

namespace Kadet\Functional\Predicate;


use function Kadet\Functional\{ all, any };
use Kadet\Functional\Predicate;

class ConstantPredicate extends AbstractPredicate
{
    /** @var boolean */
    private $value;

    /**
     * ConstantPredicate constructor.
     *
     * @param bool $value
     */
    private function __construct(bool $value)
    {
        $this->value = $value;
    }

    public function not(): Predicate
    {
        return $this->value
            ? self::never()
            : self::always();
    }

    public function and (...$predicate): Predicate
    {
        if (!$this->value) {
            return self::never();
        }

        return all($predicate);
    }

    public function or (...$predicate): Predicate
    {
        if ($this->value) {
            return self::always();
        }

        return any($predicate);
    }

    public function __invoke(...$args): bool
    {
        return $this->value;
    }

    public static function always()
    {
        static $always;
        return $always ? $always : $always = new static(true);
    }

    public static function never()
    {
        static $never;
        return $never ? $never : $never = new static(false);
    }
}