<?php
/**
 * Copyright 2018 Kacper Donat
 *
 * @author Kacper "Kadet" Donat <kacper@kadet.net>
 *
 * Full license available in separate LICENSE file
 */

namespace Kadet\Functional\Utils;


use function Kadet\Functional\reflect;

class CurriedFunction extends PartiallyAppliedFunction
{
    const INF = PHP_INT_MAX;

    private $threshold;

    /**
     * CurriedFunction constructor.
     *
     * @param callable $callable
     * @param int|null $threshold
     * @param array    $args
     */
    public function __construct(callable $callable, int $threshold = null, ...$args)
    {
        parent::__construct($callable, ...$args);
        $this->threshold = $threshold ?: reflect($callable)->getNumberOfRequiredParameters();
    }

    public function getThreshold(): int
    {
        return $this->threshold;
    }

    public function withThreshold(int $threshold = null)
    {
        return new static($this, $threshold);
    }

    public function apply(...$args)
    {
        return new static($this, $this->threshold, ...$args);
    }

    public function call(...$args)
    {
        return parent::__invoke(...$args);
    }

    public function uncurry()
    {
        return new PartiallyAppliedFunction($this);
    }

    /**
     * @param mixed ...$args
     *
     * @return \Kadet\Functional\Utils\CurriedFunction|mixed
     */
    public function __invoke(...$args)
    {
        if (count($args) === 0) {
            return $this->call();
        }

        $applied = $this->apply(...$args);
        if (count($applied->getArguments()) >= $this->threshold) {
            return $applied->call();
        }

        return $applied;
    }
}