<?php
/**
 * Copyright 2018 Kacper Donat
 *
 * @author Kacper "Kadet" Donat <kacper@kadet.net>
 *
 * Full license available in separate LICENSE file
 */

namespace Kadet\Functional\Utils;


class FunctionPipeline
{
    /**
     * @var callable[]
     */
    private $functions = [];

    /**
     * FunctionPipeline constructor.
     *
     * @param callable[] $functions
     */
    public function __construct(callable ...$functions)
    {
        $this->functions = $functions;
    }

    public function append(callable $callable)
    {
        return new static(...array_merge($this->functions, [ $callable ]));
    }

    public function prepend(callable $callable)
    {
        return new static($callable, ...$this->functions);
    }

    public function __invoke(...$args)
    {
        return array_reduce($this->functions, function ($arg, $callable) {
            return $arg instanceof UnpackedArgs
                ? $callable(...$arg->getArrayCopy())
                : $callable($arg);
        }, new UnpackedArgs($args));
    }
}